/*-
 * Copyright 2011 Jason Harmening
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifndef __CX88_DVB_BUFFER_H__
#define __CX88_DVB_BUFFER_H__

#include <stdio.h>
#include <pthread.h>

class cx88_dvb_buffer
{

   public:

      cx88_dvb_buffer(size_t size, int &error);

      virtual ~cx88_dvb_buffer(void);

      virtual int resize(size_t newsize);

      virtual int write(const void *data, size_t size);

      virtual int read(void *user_ptr, int fflags, size_t &size);

      virtual int poll(int events);

      virtual void reset(void);

      virtual void set_timeout(unsigned long timeout_s);

      void set_break(void) { m_break_pos = m_write_pos; }

      void cancel_transfer(void);

   protected:

      void reset_pointers(void);

      uint8_t *m_buffer;
      size_t m_size;
      size_t m_read_pos;
      size_t m_write_pos;
      size_t m_break_pos;
      pthread_mutex_t m_mutex;
      pthread_mutex_t m_read_lock;
      pthread_cond_t m_data_avail;
      enum
      {
         CX88_DVB_BUF_UNINITIALIZED,
         CX88_DVB_BUF_MUTEX_CREATED,
         CX88_DVB_BUF_READ_LOCK_CREATED,
         CX88_DVB_BUF_CV_CREATED
      } m_state;
      struct timespec m_deadline;
      bool m_timeout;
      bool m_write_rollover;
      bool m_canceled;
};

#endif

