/*-
 * Copyright 2010 Jason Harmening
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifndef __CX88_DVB_CAPTURE_H__
#define __CX88_DVB_CAPTURE_H__

#include <sys/types.h>
#include <list>
extern "C"
{
   #include <cuse4bsd.h>
}
#include "cx88_capture.h"
#include "cx88_dvb_frontend.h"
#include "cx88_dvb_buffer.h"
#include "cx88_dvb_demux.h"

class cx88_dvb_capture
   : public cx88_capture
{

   public:

      cx88_dvb_capture(const std::string &id, const char *devnode, int &error);

      virtual ~cx88_dvb_capture(void);

      virtual int set_driver(cx88_driver *driver)
      {
         return m_frontend.set_driver(driver);
      }

   protected:

      virtual void process_buffer(void *data, size_t num_bytes, size_t packet_size = 0)
      {
         m_demux.submit_buffer(data, num_bytes);
      }

      virtual void process_notification(uint32_t events, int error);

      enum dvb_device_t
      {
         DVB_DEVICE_TYPE_DEMUX = 0,
         DVB_DEVICE_TYPE_DVR,
         DVB_DEVICE_TYPE_FRONTEND,
         NUM_DVB_DEVICE_TYPES
      };

      static const char* m_dvb_devnames[NUM_DVB_DEVICE_TYPES];

      static const struct cuse_methods m_cuse_methods[NUM_DVB_DEVICE_TYPES];
      struct cuse_dev *m_cuse_devs[NUM_DVB_DEVICE_TYPES];

      static const size_t CX88_DVB_NUM_IO_THREADS = NUM_DVB_DEVICE_TYPES * 2;
      pthread_t m_cuse_threads[CX88_DVB_NUM_IO_THREADS];
      size_t m_num_io_threads;
      enum
      {
         CX88_DVB_CAPTURE_UNINITIALIZED,
         CX88_DVB_CAPTURE_FEED_LOCK_CREATED,
         CX88_DVB_CAPTURE_BUFFER_KEY_CREATED,
         CX88_DVB_CAPTURE_FRONTEND_KEY_CREATED,
         CX88_DVB_CAPTURE_CUSE_INITIALIZED
      } m_state;

      cx88_dvb_buffer m_dvr;
      cx88_dvb_demux  m_demux;
      cx88_dvb_frontend m_frontend;

      class demux_feed_entry
      {
         public:

            demux_feed_entry(int &error) : m_feed(error), m_prev(NULL), m_next(NULL) {}

            ~demux_feed_entry(void) {}

            void insert(demux_feed_entry *prev, demux_feed_entry *next);

            void remove(void);

            demux_feed_entry *get_next(void) { return m_next; }

            demux_feed_entry *get_prev(void) { return m_prev; }

            cx88_dvb_demux_feed &get_feed(void) { return m_feed; }

         private:

            cx88_dvb_demux_feed m_feed;
            demux_feed_entry *m_prev;
            demux_feed_entry *m_next;
      };

      demux_feed_entry *m_feeds;
      pthread_mutex_t m_feed_lock;

      static void *cuse_thread_routine(void *arg);

      static int dvr_open(struct cuse_dev *dev, int fflags);

      static int dvr_read(struct cuse_dev *dev, int fflags, void *user_ptr, int len);

      static int dvr_ioctl(struct cuse_dev *dev, int fflags, unsigned long cmd, void *user_data);

      static int dvr_poll(struct cuse_dev *dev, int fflags, int events);

      static int demux_open(struct cuse_dev *dev, int fflags);

      static int demux_close(struct cuse_dev *dev, int fflags);

      static int demux_read(struct cuse_dev *dev, int fflags, void *user_ptr, int len);

      static int demux_ioctl(struct cuse_dev *dev, int fflags, unsigned long cmd, void *user_data);

      static int demux_poll(struct cuse_dev *dev, int fflags, int events);

      static int frontend_open(struct cuse_dev *dev, int fflags);

      static int frontend_close(struct cuse_dev *dev, int fflags);

      static int frontend_ioctl(struct cuse_dev *dev, int fflags, unsigned long cmd, void *user_data);

      static int frontend_poll(struct cuse_dev *dev, int fflags, int events);

};

#endif

