/*-
 * Copyright 2011 Jason Harmening
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifndef __CX88_DVB_DEMUX_H__
#define __CX88_DVB_DEMUX_H__

#include <list>
#include <map>
#include "cx88_dvb_demux_feed.h"

class cx88_dvb_demux
{
   public:

      cx88_dvb_demux(cx88_dvb_buffer &dvr, int &error);

      virtual ~cx88_dvb_demux(void);

      int submit_buffer(void *buffer, size_t size);

      int ioctl(cx88_dvb_demux_feed &feed, unsigned long code, void *user_data);

   protected:

      int add_feed(uint16_t pid, cx88_dvb_demux_feed &feed);

      void remove_feed(uint16_t pid, cx88_dvb_demux_feed &feed);

      int add_feed(cx88_dvb_demux_feed &feed);

      void remove_feed(cx88_dvb_demux_feed &feed);

      uint8_t *get_packet(uint8_t *&buffer, size_t &size);

      cx88_dvb_buffer &m_dvr;
      pthread_mutex_t m_feed_lock;
      bool m_feed_lock_created;

      typedef std::list<cx88_dvb_demux_feed*> demux_feed_list;
      struct demux_pid_filter
      {
         demux_pid_filter(void) : continuity(0), invalidate(false), initialized(false) {}
         demux_feed_list feeds;
         uint8_t continuity;
         bool invalidate;
         bool initialized;
      };

      typedef std::map<uint16_t, demux_pid_filter> demux_filter_map;
      demux_filter_map m_filters;

      uint8_t m_last_packet[188];
      uint8_t m_last_packet_offset;
};

#endif

