/*-
 * Copyright 2011 Jason Harmening
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifndef __CX88_DVB_DEMUX_FEED_H__
#define __CX88_DVB_DEMUX_FEED_H__

#include <set>
#include <sys/ioccom.h>
#include <linux/dvb/dmx.h>
#include "cx88_dvb_buffer.h"

class cx88_dvb_demux_feed
   : public cx88_dvb_buffer
{
   public:

      cx88_dvb_demux_feed(int &error);

      virtual ~cx88_dvb_demux_feed(void);

      virtual void reset(void);

      void invalidate(void);

      int set_filter(struct dmx_sct_filter_params &params);

      int set_filter(struct dmx_pes_filter_params &params);

      int add_pid(uint16_t pid);

      void remove_pid(uint16_t pid) { m_pids.erase(pid); }

      int start(void);

      void stop(void);

      enum demux_filter_state
      {
         CX88_DVB_DEMUX_FILTER_UNCONFIGURED,
         CX88_DVB_DEMUX_FILTER_STOPPED,
         CX88_DVB_DEMUX_FILTER_RUNNING
      };

      demux_filter_state get_state(void) { return m_state; }

      enum demux_filter_result
      {
         DEMUX_FILTER_REJECT,
         DEMUX_FILTER_ACCEPT,
         DEMUX_FILTER_FINISHED
      };

      demux_filter_result filter_packet(
         uint8_t *packet,
         size_t payload_offset,
         size_t &crc_extent,
         size_t &crc_passed,
         size_t packet_size = 188);

      typedef std::set<uint16_t> pid_list;
      pid_list &get_pids(void) { return m_pids; }

      enum demux_feed_type
      {
         CX88_DVB_DEMUX_FEED_NONE,
         CX88_DVB_DEMUX_FEED_SECTION,
         CX88_DVB_DEMUX_FEED_PES
      };

      enum demux_output_type
      {
         CX88_DVB_DEMUX_OUTPUT_DVR,
         CX88_DVB_DEMUX_OUTPUT_TSDEMUX,
         CX88_DVB_DEMUX_OUTPUT_PAYLOAD,
         CX88_DVB_DEMUX_OUTPUT_NONE
      };

      demux_feed_type get_type(void) { return m_type; }

      demux_output_type get_output(void) { return m_output; }

   protected:

      static uint32_t crc32(uint8_t *packet, size_t size);

      int set_filter(uint16_t pid, uint32_t flags);

      demux_filter_result secbuf_write(uint8_t *packet,
                                       size_t offset,
                                       size_t size,
                                       size_t &crc_extent,
                                       size_t &crc_passed);

      uint8_t m_secbuf[4096 + 188];
      uint16_t m_secbuf_start;
      uint16_t m_secbuf_pos;
      demux_filter_state m_state;
      demux_feed_type m_type;
      demux_output_type m_output;
      pid_list m_pids;
      uint32_t m_flags;
      uint8_t m_filter[DMX_FILTER_SIZE];
      uint8_t m_eq_mask[DMX_FILTER_SIZE];
      uint8_t m_neq_mask[DMX_FILTER_SIZE];
      bool m_neq;
      bool m_reject;
};

#endif
