/*-
 * Copyright 2011 Jason Harmening
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifndef __CX88_DVB_FRONTEND_H__
#define __CX88_DVB_FRONTEND_H__

#include <pthread.h>
#include <time.h>
#include <queue>
#include <sys/ioccom.h>
#include <linux/dvb/frontend.h>
#include "cx88_mpeg_driver.h"

class cx88_dvb_frontend
{

   public:

      cx88_dvb_frontend(int &error);

      virtual ~cx88_dvb_frontend(void);

      int start(void);

      void stop(void);

      void cancel_io(void);      

      int set_driver(cx88_driver *driver);

      void channel_changed(int error);

      int ioctl(int fflags, unsigned long code, void *user_data);

      int open(void);

      void close(void);

      int poll(int events);

   protected:

      void get_event(struct dvb_frontend_event &event)
      {
         event.parameters = m_params;
      }
      
      template <typename dvT, typename ltT, size_t n>
      static ltT dvb_to_libtuner(const ltT (&arr)[n], dvT index, int &error);

      int set_params(dvb_channel channel, struct dvb_frontend_parameters &params);

      int set_properties(struct dtv_property *props, size_t num_props);

      int get_properties(struct dtv_property *props, size_t num_props);

      int get_signal(void);

      cx88_mpeg_driver *m_driver;
      uint32_t m_refcount;
      pthread_mutex_t m_lock;
      pthread_cond_t m_event;
      enum
      {
         CX88_FRONTEND_UNINITIALIZED,
         CX88_FRONTEND_MUTEX_CREATED,
         CX88_FRONTEND_CV_CREATED,
         CX88_FRONTEND_STOPPED,
         CX88_FRONTEND_CONFIGURED,
         CX88_FRONTEND_ERROR,
         CX88_FRONTEND_TUNED
      } m_state;
      struct dvb_frontend_parameters m_params;
      struct dvb_frontend_info m_info;
      struct dvb_frontend_parameters m_param_cache;
      dvb_channel m_properties;
      dvb_signal m_signal;
      time_t m_signal_timestamp;
      std::deque<struct dvb_frontend_event> m_events;
      bool m_oneshot;
      bool m_opened;
      bool m_canceled;
};

#endif
